-- Addon:	LockoutChecker
-- Author:	Spielstein@Curse


if GetLocale() ~= "enGB" and GetLocale() ~= "enUS" and GetLocale() ~= "deDE" then
	return
end


------------------------------------------
--		Variables, Constants			--
--		and Meta Functions				--
------------------------------------------


-- Function for Creating a set out of an array
function LockoutChecker:GetSet(list)

	local _
	local l
	local set = {}
	for _, l in ipairs(list) do
		set[l] = true
	end
	return set

end


-- Creating a set of all english legion dungeon names
LockoutChecker.EnglishLegionDungeonsNonSet = {
	"Eye of Azshara", "Halls of Valor", "The Arcway", "Maw of Souls",
	"Darkheart Thicket", "Court of Stars", "Vault of the Wardens",
	"Assault on Violet Hold", "Black Rook Hold", "Neltharion's Lair"}
LockoutChecker.EnglishLegionDungeons = LockoutChecker:GetSet(LockoutChecker.EnglishLegionDungeonsNonSet)


-- Creating a set of all german legion dungeon names
LockoutChecker.GermanLegionDungeonsNonSet = {
	"Das Auge Azsharas", "Finsterherzdickicht", "Hallen der Tapferkeit",
	"Neltharions Hort", "Violette Festung", "Das Verlies der Wächterinnen",
	"Rabenwehr", "Der Seelenschlund", "Der Hof der Sterne", "Der Arkus"}
LockoutChecker.GermanLegionDungeons = LockoutChecker:GetSet(LockoutChecker.GermanLegionDungeonsNonSet)

LockoutChecker.ENtoDE = {
	["Eye of Azshara"] = "Das Auge Azsharas",
	["Halls of Valor"] = "Hallen der Tapferkeit",
	["The Arcway"] = "Der Arkus",
	["Maw of Souls"] = "Der Seelenschlund",
	["Darkheart Thicket"] = "Finsterherzdickicht",
	["Court of Stars"] = "Der Hof der Sterne",
	["Vault of the Wardens"] = "Das Verlies der Wächterinnen",
	["Assault on Violet Hold"] = "Violette Festung",
	["Black Rook Hold"] = "Rabenwehr",
	["Neltharion's Lair"] = "Neltharions Hort"
}

LockoutChecker.DEtoEN = {
	["Das Auge Azsharas"] = "Eye of Azshara",
	["Finsterherzdickicht"] = "Darkheart Thicket",
	["Hallen der Tapferkeit"] = "Halls of Valor",
	["Neltharions Hort"] = "Neltharion's Lair",
	["Violette Festung"] = "Assault on Violet Hold",
	["Das Verlies der Wächterinnen"] = "Vault of the Wardens",
	["Rabenwehr"] = "Black Rook Hold",
	["Der Seelenschlund"] = "Maw of Souls",
	["Der Hof der Sterne"] = "Court of Stars",
	["Der Arkus"] = "The Arcway"
}


--
function LockoutChecker:Translate(text, langCode)

	local result = nil

	if langCode == "deDE" then
		return LockoutChecker.DEtoEN[text]
	else
		return LockoutChecker.ENtoDE[text]
	end
end


-- Locale wrapper
function LockoutChecker:GetLocalizedData(selection)
	local language = GetLocale()
	if language == "deDE" then
		if selection == "set" then
			return LockoutChecker.GermanLegionDungeons
		elseif selection == "list" then
			return LockoutChecker.GermanLegionDungeonsNonSet
		else
			return nil
		end
	else
		if selection == "set" then
			return LockoutChecker.EnglishLegionDungeons
		elseif selection == "list" then
			return LockoutChecker.EnglishLegionDungeonsNonSet
		else
			return nil
		end
	end
end


------------------------------------------
--		Addon Functions					--
------------------------------------------


-- Function for checking whether a dungeon name belongs to a legion dungeon
function LockoutChecker:IsLegionDungeon(dungeonName)

	if LockoutChecker:GetLocalizedData("set")[dungeonName]then
		return true
	else
		return false
	end

end


-- Function for checking for lockouts
function LockoutChecker:SearchForLockouts()

	-- Variables --

	-- For-loop integer
	local i, j
	-- Bucket variable
	local _

	-- Bariables for the GetSavedInstanceInfo(i) api-call
	local name, id, difficulty, locked, extended, isRaid, difficultyName, numEncounters
	-- Bariables for the GetSavedInstanceEncounterInfo(i,j) API call
	local bossName, isKilled

	-- Variable for formatting string later
	local formattedBossName

	-- Storage for remembering dungeons with a lock
	local dungeonsWithLocks = {}
	-- Same as >> dungeonsWithLocks << but as a set
	local dungeonsWithLocksAsSet = {}
	-- Variables for interating through the table >> dungeonsWithLocks <<
	local dName, dBosses


	-- Iterating through all saved instance ids of a char
	for i = 1, GetNumSavedInstances() do

		-- API call
		name, id, _, difficulty, locked, extended, _, isRaid, _, difficultyName, numEncounters, _ = GetSavedInstanceInfo(i)

		-- If dungeon is a mythic dungeon, a legion dungeon and as a lock or is extended
		if difficulty == 23 and LockoutChecker:IsLegionDungeon(name) and (locked or extended) then
		
			-- Remember locked dungeon for later
			table.insert(dungeonsWithLocks, name)

			formattedBossName = ""

			-- Iterate through all encounter in the dungeon
			for j = 1, numEncounters do

				-- API call
				bossName, _, isKilled, _ = GetSavedInstanceEncounterInfo(i, j)
				-- If the encounter is killed, give the encounter name a red color, green otherwise
				formattedBossName = formattedBossName .. (isKilled and "|cffff0000" or "|cff008000") .. bossName .. ((j ~= numEncounters) and "\n" or "")
			end
			LockoutChecker.gui.fields[2][name]:SetText(formattedBossName:format())
		end
	end

	dungeonsWithLocksAsSet = LockoutChecker:GetSet(dungeonsWithLocks)
	
	-- TODO: german local
	-- Interate through all english dungeon names
	for dName, _ in pairs(LockoutChecker:GetLocalizedData("set")) do
		-- If dungeon as no lockout ...
		if not dungeonsWithLocksAsSet[dName] then
			-- ... color the dungeon name green
			local output = "|cff008000" .. dName
			-- Print the colored dungeon name
			LockoutChecker.gui.fields[2][dName]:SetText(output:format())
		end
	end

	local colorStr = RAID_CLASS_COLORS[select(2, UnitClass("player"))]["colorStr"]
	LockoutChecker.gui.fields[2]["cNames"]:SetText(string.format("|c%s%s", colorStr, LockoutChecker.ownName))

end


--
function LockoutChecker:CheckForGroupMembers()

	LockoutChecker.partyPeople = {}

	local i

	local max = GetNumGroupMembers(LE_PARTY_CATEGORY_HOME)-1

	for i = 1, max do
		local colorStr = RAID_CLASS_COLORS[select(2, UnitClass("party" .. i))]["colorStr"]
		local name, realm = UnitName("party" .. i)
		name = name.."-"..((realm ~= nil and realm ~= "") and realm or GetRealmName())
		LockoutChecker.gui.fields[i+2]["cNames"]:SetText(string.format("|c%s%s", colorStr, name))
		LockoutChecker.partyPeople[name] = i+2
	end
end


--
function LockoutChecker:ClearMatrix()
	local _, v, i
	for i = 2, 6 do
		for _, v in ipairs(LockoutChecker:GetLocalizedData("list")) do
			LockoutChecker.gui.fields[i][v]:SetText("")
		end
		LockoutChecker.gui.fields[i]["cNames"]:SetText("")
	end
end

function LockoutChecker:ButtonUpdate(elapsed)

	LockoutChecker.gui.syncButton.elapsedTime = LockoutChecker.gui.syncButton.elapsedTime + elapsed

    if LockoutChecker.gui.syncButton.elapsedTime >= LockoutChecker.gui.syncButton.maxTime then

    	LockoutChecker.gui.syncButton:SetText("Sync")
        LockoutChecker.gui.syncButton:Enable()
        LockoutChecker.gui.syncButton.elapsedTime = 0
        LockoutChecker.gui.syncButton:SetScript("OnUpdate", nil)

    else
    	LockoutChecker.gui.syncButton:SetText(string.format("%.0f", LockoutChecker.gui.syncButton.maxTime - LockoutChecker.gui.syncButton.elapsedTime))
    end
end


--
function LockoutChecker:SendRequest()
	SendAddonMessage("LockoutChecker", "Request", "PARTY" )
end


--
function LockoutChecker:SendData(requester)
	-- Variables --

	-- For-loop integer
	local i, j
	-- Bucket variable
	local _

	-- Bariables for the GetSavedInstanceInfo(i) api-call
	local name, id, difficulty, locked, extended, isRaid, difficultyName, numEncounters
	-- Bariables for the GetSavedInstanceEncounterInfo(i,j) API call
	local bossName, isKilled

	-- Variable for formatting string later
	local formattedBossName

	-- Storage for remembering dungeons with a lock
	local dungeonsWithLocks = {}
	-- Same as >> dungeonsWithLocks << but as a set
	local dungeonsWithLocksAsSet = {}
	-- Variables for interating through the table >> dungeonsWithLocks <<
	local dName, dBosses


	-- Iterating through all saved instance ids of a char
	for i = 1, GetNumSavedInstances() do

		-- API call
		name, id, _, difficulty, locked, extended, _, isRaid, _, difficultyName, numEncounters, _ = GetSavedInstanceInfo(i)

		-- If dungeon is a mythic dungeon, a legion dungeon and as a lock or is extended
		if difficulty == 23 and LockoutChecker:IsLegionDungeon(name) and (locked or extended) then
		
			-- Remember locked dungeon for later
			table.insert(dungeonsWithLocks, name)

			-- Iterate through all encounter in the dungeon
			for j = 1, numEncounters do

				-- API call
				bossName, _, isKilled, _ = GetSavedInstanceEncounterInfo(i, j)
				-- If the encounter is killed, give the encounter name a red color, green otherwise
				formattedBossName = (isKilled and "|cffff0000" or "|cff008000") .. bossName .. ((j ~= numEncounters) and "\n" or "")
				SendAddonMessage("LockoutChecker", requester..":"..  GetLocale() ..":"..name..":"..formattedBossName, "PARTY")
			end
		end
	end

	dungeonsWithLocksAsSet = LockoutChecker:GetSet(dungeonsWithLocks)
	
	-- TODO: german local
	-- Interate through all english dungeon names
	for dName, _ in pairs(LockoutChecker:GetLocalizedData("set")) do
		-- If dungeon as no lockout ...
		if not dungeonsWithLocksAsSet[dName] then
			-- ... color the dungeon name green
			local output = "|cff008000" .. dName
			-- Print the colored dungeon name
			SendAddonMessage("LockoutChecker", requester..":"..  GetLocale() ..":"..dName..":"..string.format(output), "PARTY")
		end
	end
end


--
function LockoutChecker:CollectData(data, dataSource)
	
	local dataAsList = {}
	local token
	for token in string.gmatch(data, "[^:]+") do
		table.insert(dataAsList, token)
	end

	if dataAsList[1] == LockoutChecker.ownName then
		local playerID = LockoutChecker.partyPeople[dataSource]
		local language = dataAsList[2]
		local dungeon = dataAsList[3]
		local text = dataAsList[4]

		if dataAsList[2] ~= GetLocale() then
			dungeon = LockoutChecker:Translate(dungeon, language)
		end
		
		LockoutChecker.gui.fields[playerID][dungeon]:Append(text)

		
	end
end


--
function LockoutChecker:EventHandler(event, ...)
	if (event == "CHAT_MSG_ADDON" and select(1,...) == "LockoutChecker" and select(4,...) ~= LockoutChecker.ownName) then
        
        local args = {...}
        
        if args[2] == "Request" then
        	LockoutChecker:SendData(args[4])
        else
        	LockoutChecker:CollectData(args[2], args[4])
        end
    end
end


--
function LockoutChecker:SyncButtonPressed()

	LockoutChecker.gui.syncButton:Disable()
	LockoutChecker.gui.syncButton:SetScript("OnUpdate", LockoutChecker.ButtonUpdate)

	LockoutChecker:ClearMatrix()

	if GetNumGroupMembers(LE_PARTY_CATEGORY_HOME) ~= 0 then
		-- PlayceHolder
		LockoutChecker:SearchForLockouts()
		LockoutChecker:CheckForGroupMembers()
		LockoutChecker:SendRequest()
	else
		LockoutChecker:SearchForLockouts()
	end
end


------------------------------------------
--		StartUp							--
------------------------------------------


LockoutChecker.gui = LockoutChecker:CreateGUI(UIParent, LockoutChecker:GetLocalizedData("list"), LockoutChecker.SyncButtonPressed)

LockoutChecker.gui:RegisterEvent("CHAT_MSG_ADDON");
LockoutChecker.gui:SetScript("OnEvent", LockoutChecker.EventHandler)
RegisterAddonMessagePrefix("LockoutChecker")

local _, v

for _, v in ipairs(LockoutChecker:GetLocalizedData("list")) do
	LockoutChecker.gui.fields[1][v]:SetText(v)
end

LockoutChecker.ownName, LockoutChecker.ownRealm = UnitName("player")
LockoutChecker.ownName = LockoutChecker.ownName.."-"..((LockoutChecker.ownRealm ~= nil and LockoutChecker.ownRealm ~= "") and LockoutChecker.ownRealm or GetRealmName())


------------------------------------------
--		SlashCommands					--
------------------------------------------


-- Slash command #1
SLASH_LockoutChecker1 = "/lc"
-- Slash command #2
SLASH_LockoutChecker2 = "/LockoutChecker"

-- Register slash commands
SlashCmdList["LockoutChecker"] = function(msg)
    if LockoutChecker.gui:IsShown() then
    	LockoutChecker.gui:Hide()
    else
    	LockoutChecker.gui:Show()
    end
end


------------------------------------------
--		End of Script					--
------------------------------------------